class OrderCycle(object):
    """ 
        # Order Cycle

        Use in conjucntion with the rest of the pipeline.

        1. Create orders
        2. Change status of orders
        3. Finish orders
        4. Get history of individual order
    """
    def __init__(self):
        pass
    pass