class RLEnvironment(object):
    """ Use to setup and start a reinforcement learning training session """
    def __init__(self, *args, **kwargs):
        # Set strategy
        # Set multidataset
        # Set time
        pass
    
    def time(self):
        pass

    def requirements(self):
        """ Creates a requirements handler based on """
        pass